# Arduino et le module RTC
![category](domotique)
![tag](electronique)
![tag](arduino)

Si arduino connait la notion de temps et sait faire une pause d'une seconde

```c++
delay(1000)
```

Il ne connait pas l'heure. Pire si on le débranche il perd toutes notions de temps
écoulés.

Pour régler ce problème il existe des modules permettant de gérer l'heure.
Il s'agit de module RTC.

Ils sont composés d'une pile permettant de retenir la notion de temps écoulé
même quand l'Arduino est éteint.

Pour fonctionner il a besoin d'être alimenté.

Le module que j'ai choisit est un module RTC DS1307 mode grove (modèle de connectique)
http://wiki.seeed.cc/Grove-RTC/.
Il fonctionne  en I2C: il s'agit d'un bus bidirectionnel permet d'initier l'heure
et de fournir l'information de temps.

Il comporte 4 broches:

- GND ... à placer sur un pin GND de l'arduino
- VCC ... à placer sur un pin 5V de l'arduino
- SDA (ligne de données) ... à placer sur le pin SDA de l'arduino
- SCL (ligne de l'horloge) ... à placer sur le pin SCL de l'arduino

![201702211850_01.png](./upload/201702211850_01.png)

Utilisation classique
---------------------

Pour fonctionner il faut utiliser de librairie:

- librairie *Wire.h* pour le bus I2C
- librairie *DS1307.h* pour communiquer avec le module

La librairie *Wire.h* est intégré dans l'arduino IDE

On peut trouver la librairie *DS1307.h* sur le lien https://raw.githubusercontent.com/SeeedDocument/Grove-RTC/master/res/RTC_Library.zip
(il suffira de copier les fichier DS1307.h et DS1307.cpp au même niveau que notre
code)

Et enfin notre code

```c++
#include <Wire.h>
#include "DS1307.h"

DS1307 clock;//define a object of DS1307 class
String daysOfWeek[] = {"LUN", "MAR", "MER", "JEU", "VEN", "SAM", "DIM"};
String libOfMonths[] = {"JAN", "FEV", "MAR", "AVR", "MAI", "JUI", "JUI", "AOU", "SEP", "OCT", "NOV", "DEC"};

void setup()
{
    Serial.begin(9600);
    clock.begin();
    clock.getTime();
    if (clock.year == 0 )
    {
        clock.fillByYMD(2016,2,21);
        clock.fillByHMS(19,30,22);
        clock.fillDayOfWeek(TUE);//Tuesday list of day MON,TUE,WED,THU,FRI,SAT,SUN
        clock.setTime();//write time to the RTC chip
    }
}
void loop()
{
    printTime();
}

/*Function: Display time on the serial monitor*/
void printTime()
{
    clock.getTime();
    Serial.print(daysOfWeek[clock.dayOfWeek - 1]);
    Serial.print(" ");
    Serial.print(clock.dayOfMonth, DEC);
    Serial.print(" ");
    Serial.print(libOfMonths[clock.month - 1]);
    Serial.print(" ");
    Serial.print(clock.year+2000, DEC);
    Serial.print(" ");
    Serial.print(clock.hour, DEC);
    Serial.print(":");
    Serial.print(clock.minute, DEC);
    Serial.print(":");
    Serial.print(clock.second, DEC);
    Serial.print(" ");
    Serial.println(" ");
    delay(1000);
}
```

Une fois uploader sur l'Arduino on peut voir sur la sortie *Série* la date 
s'afficher.

On note que dans la partie Setup, si l'horloge n'a jamais été initialiser (année 0)
alors je charge l'heure souhaitée.

Synchronisation avec le PC
--------------------------

Il est possible via une autre librairie d'obtenir une synchronisation de l'heure
de l'ardunio avec l'heure de compilation du programme.

Pour cela j'utilise la librairie RTCLib disponible sur https://github.com/adafruit/RTClib.

le code correspondant est le suivant

```c++ 
#include <Wire.h>
#include "RTClib.h"
 
RTC_DS1307 RTC; //Classe RTC_DS1307

void setup()
{
    Serial.begin(9600);
    RTC.begin();
    DateTime now = RTC.now();
    if (now.year() == 2000 )
    {      
        RTC.adjust(DateTime(__DATE__, __TIME__)); 
    }
}
void loop()
{
    printTime();
}
    /*Function: Display time on the serial monitor*/
void printTime()
{
    DateTime now = RTC.now();
    Serial.print(now.day(), DEC);
    Serial.print('/');
    Serial.print(now.month(), DEC);
    Serial.print('/');
    Serial.print(now.year(), DEC);  
    Serial.print(' ');
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.print(now.second(), DEC);
    Serial.println();
    delay(1000);
}

```
